@@ -11,6 +11,15 @@
 #define HAVE_EXPLICIT_MEMSET 0
 #define HAVE_CLOCK_NANOSLEEP 1
 #define HAVE_SIGACTION 1
+#define WANT_THREADS 0
+#define WANT_CAPSTONE 1
+
+#define HAVE_GPERF 0
+#if (HAVE_GPERF) == 1
+#define HAVE_GPERF 0
+#else
+#define HAVE_GPERF 0
+#endif
 
 #ifdef R_MESON_VERSION
 #define R2_PREFIX "/usr"
@@ -18,17 +27,17 @@
 #define R2_LIBDIR "/usr/lib"
 #define R2_INCDIR "/usr/include/libr"
 #define R2_DATDIR "/usr/share"
-#define R2_WWWROOT "/usr/share/radare2/5.2.1/www"
-#define R2_PLUGINS "lib/radare2/5.2.1"
-#define R2_EXTRAS "lib/radare2-extras/5.2.1"
-#define R2_BINDINGS "lib/radare2-bindings/5.2.1"
+#define R2_WWWROOT "/usr/share/radare2/5.6.6/www"
+#define R2_PLUGINS "lib/radare2/5.6.6"
+#define R2_EXTRAS "lib/radare2-extras/5.6.6"
+#define R2_BINDINGS "lib/radare2-bindings/5.6.6"
 #define R2_DATDIR_R2 "share/radare2"
-#define R2_SDB "share/radare2/5.2.1"
-#define R2_ZIGNS "share/radare2/5.2.1/zigns"
-#define R2_THEMES "share/radare2/5.2.1/cons"
+#define R2_SDB "share/radare2/5.6.6"
+#define R2_ZIGNS "share/radare2/5.6.6/zigns"
+#define R2_THEMES "share/radare2/5.6.6/cons"
 #define R2_FORTUNES "share/doc/radare2"
-#define R2_FLAGS "share/radare2/5.2.1/flag"
-#define R2_HUD "share/radare2/5.2.1/hud"
+#define R2_FLAGS "share/radare2/5.6.6/flag"
+#define R2_HUD "share/radare2/5.6.6/hud"
 #else
 #if __WINDOWS__ || _MSC_VER
 #define R2_PREFIX "."
@@ -98,7 +107,12 @@
 
 #define HAVE_LIBUV 1
 
+#if __MINGW32__
+#define HAVE_PTRACE 0
+#else
 #define HAVE_PTRACE 1
+#endif
+
 #define USE_PTRACE_WRAP 1
 #define HAVE_FORK 1
 #define WANT_DYLINK 1
