@@ -32,8 +32,8 @@ typedef struct {
 R_API RSysInfo *r_sys_info(void);
 R_API void r_sys_info_free(RSysInfo *si);
 
-R_API int r_sys_sigaction(int *sig, void (*handler) (int));
-R_API int r_sys_signal(int sig, void (*handler) (int));
+R_API int r_sys_sigaction(int *sig, void(*handler)(int));
+R_API int r_sys_signal(int sig, void(*handler)(int));
 R_API void r_sys_env_init(void);
 R_API char **r_sys_get_environ(void);
 R_API void r_sys_set_environ(char **e);
@@ -106,7 +106,7 @@ R_API bool r_sys_tts(const char *txt, bool bg);
 #if __WINDOWS__
 #  define r_sys_breakpoint() { __debugbreak  (); }
 #else
-#if __GNUC__
+#if __GNUC__ && !defined(__TINYC__)
 #  define r_sys_breakpoint() __builtin_trap()
 #elif __i386__ || __x86_64__
 #   define r_sys_breakpoint() __asm__ volatile ("int3");
@@ -121,6 +121,8 @@ R_API bool r_sys_tts(const char *txt, bool bg);
 #  endif
 #elif __mips__
 #  define r_sys_breakpoint() __asm__ volatile ("break");
+#elif __loongarch__
+#  define r_sys_breakpoint() __asm__ volatile ("break");
 // #  define r_sys_breakpoint() __asm__ volatile ("teq $0, $0");
 #elif __EMSCRIPTEN__
 // TODO: cannot find a better way to breakpoint in wasm/asm.js
@@ -133,7 +135,7 @@ R_API bool r_sys_tts(const char *txt, bool bg);
 #endif
 
 /* syscmd */
-R_API char *r_syscmd_ls(const char *input);
+R_API char *r_syscmd_ls(const char *input, int w);
 R_API char *r_syscmd_cat(const char *file);
 R_API char *r_syscmd_mkdir(const char *dir);
 R_API bool r_syscmd_mv(const char *input);
@@ -143,7 +145,7 @@ R_API char *r_syscmd_tail(const char *file, int count);
 R_API char *r_syscmd_join(const char *file1, const char *file2);
 R_API char *r_syscmd_sort(const char *file);
 
-R_API ut8 *r_sys_unxz (const ut8 *data, size_t len, size_t *olen);
+R_API ut8 *r_sys_unxz(const ut8 *data, size_t len, size_t *olen);
 R_API bool r_w32_init(void);
 
 #ifdef __cplusplus
