@@ -24,6 +24,9 @@ extern "C" {
 #define SDB_MIN_KEY 1
 #define SDB_MAX_KEY 0xff
 
+// ftp://ftp.gnu.org/old-gnu/Manuals/gperf-2.7/html_node/gperf_17.es.html
+#define SDB_MAX_GPERF_KEYS 15000
+
 #if !defined(SZT_ADD_OVFCHK)
 #define SZT_ADD_OVFCHK(x, y) ((SIZE_MAX - (x)) <= (y))
 #endif
@@ -81,10 +84,12 @@ extern char *strdup (const char *);
 #define SDB_KSZ 0xff
 #define SDB_VSZ 0xffffff
 
+typedef int (*GperfForeachCallback)(void *user, const char *k, const char *v);
 typedef struct sdb_gperf_t {
 	const char *name;
 	const char *(*get)(const char *k);
 	unsigned int *(*hash)(const char *k);
+	bool (*foreach)(GperfForeachCallback cb, void *user);
 } SdbGperf;
 
 typedef struct sdb_t {
@@ -149,7 +154,7 @@ SDB_API SdbList *sdb_foreach_list(Sdb* s, bool sorted);
 SDB_API SdbList *sdb_foreach_list_filter(Sdb* s, SdbForeachCallback filter, bool sorted);
 SDB_API SdbList *sdb_foreach_match(Sdb* s, const char *expr, bool sorted);
 
-SDB_API int sdb_query(Sdb* s, const char *cmd);
+SDB_API bool sdb_query(Sdb* s, const char *cmd);
 SDB_API int sdb_queryf(Sdb* s, const char *fmt, ...);
 SDB_API int sdb_query_lines(Sdb *s, const char *cmd);
 SDB_API char *sdb_querys(Sdb* s, char *buf, size_t len, const char *cmd);
@@ -243,11 +248,10 @@ SDB_API bool sdb_journal_unlink(Sdb *s);
 /* numeric */
 SDB_API char *sdb_itoa(ut64 n, char *s, int base);
 SDB_API ut64  sdb_atoi(const char *s);
-SDB_API const char *sdb_itoca(ut64 n);
 
 /* locking */
 SDB_API bool sdb_lock(const char *s);
-SDB_API const char *sdb_lock_file(const char *f);
+SDB_API bool sdb_lock_file(const char *f, char *buf, size_t buf_size);
 SDB_API void sdb_unlock(const char *s);
 SDB_API bool sdb_unlink(Sdb* s);
 SDB_API int sdb_lock_wait(const char *s UNUSED);
@@ -363,7 +367,6 @@ SDB_API char *sdb_array_pop_tail(Sdb *s, const char *key, ut32 *cas);
 
 typedef void (*SdbHook)(Sdb *s, void *user, const char *k, const char *v);
 
-SDB_API void sdb_global_hook(SdbHook hook, void *user);
 SDB_API bool sdb_hook(Sdb* s, SdbHook cb, void* user);
 SDB_API bool sdb_unhook(Sdb* s, SdbHook h);
 SDB_API int sdb_hook_call(Sdb *s, const char *k, const char *v);
@@ -384,7 +387,6 @@ SDB_API void sdb_encode_raw(char *bout, const ut8 *bin, int len);
 SDB_API int sdb_decode_raw(ut8 *bout, const char *bin, int len);
 
 // binfmt
-SDB_API char *sdb_fmt(const char *fmt, ...) SDB_PRINTF_CHECK(1, 2);
 SDB_API int sdb_fmt_init(void *p, const char *fmt);
 SDB_API void sdb_fmt_free(void *p, const char *fmt);
 SDB_API int sdb_fmt_tobin(const char *_str, const char *fmt, void *stru);
